import React, { useMemo } from 'react';
import { scaleLinear, scaleBand, scaleTime, timeFormat } from 'd3';

export type AxisComponentProps = {
  orientation: 'horizontal' | 'vertical';
  type: 'numerical' | 'categorical' | 'datetime';
  range: number[];
  domain: (number | string | Date)[];
  horizontalPosition: number;
  verticalPosition: number;
};

type Tick = { value: number | string | Date; offset: number; formattedValue?: string };

// Constants
const fontSize = 10; // Base font size
const tickLineLength = fontSize * 0.6; 
const tickTextOffsetX = -fontSize; 
const tickTextOffsetY = fontSize * 0.3; 
const horizontalTextYOffset = fontSize * 2;
const rotationAngle = 45; // Rotation angle for horizontal axis ticks
const pixelsPerTick = 40; // Pixels per tick for numerical scale
const dateFormat = timeFormat('%b'); // Change '%b' to your desired format

const AxisComponent: React.FC<AxisComponentProps> = ({ orientation, type, range, domain, verticalPosition, horizontalPosition }) => {
  const isVertical = orientation === 'vertical'; // Check if it's a vertical axis

  const ticks: Tick[] = useMemo(() => {
    if (type === 'numerical') {
      const scale = scaleLinear()
        .domain(domain as number[])
        .range(range);
      const width = Math.abs(range[1] - range[0]);
      const numberOfTicksTarget = Math.max(1, Math.floor(width / pixelsPerTick));
      
      return scale.ticks(numberOfTicksTarget).map(value => ({
        value,
        offset: scale(value),
      }));
    } else if (type === 'categorical') {
      const scale = scaleBand<string>()
        .domain(domain.map(String))
        .range(range);
        // .padding(0.1) // optional padding between bands
        return scale.domain().map(value => ({
          value,
          offset: (scale(value) ?? 0) + (scale.bandwidth() / 2),
        }));
    } else if (type === 'datetime') {
      const scale = scaleTime().domain(domain as Date[]).range(range);
      const width = Math.abs(range[1] - range[0]);
      const numberOfTicksTarget = Math.max(1, Math.floor(width / 80)); // Adjust as needed
      return scale.ticks(numberOfTicksTarget).map(value => ({
        value,
        offset: scale(value),
        formattedValue: dateFormat(value),
      }));
    }
    return [];
  }, [type, domain, range, isVertical]);

  return (
    <g>
      <g transform={orientation === 'horizontal' ? `translate(0, ${horizontalPosition})` : `translate(0, 0)`}>
        <path
          d={isVertical ? [
            "M", 0, range[0],
            "h", 0,
            "V", range[1],
            "h", -tickLineLength,
          ].join(" ") : [
            "M", range[0], tickLineLength,
            "v", -tickLineLength,
            "H", range[1],
            "v", tickLineLength,
          ].join(" ")}
          fill="none"
          stroke="currentColor"
        />
        {ticks.map(({ value, offset, formattedValue }) => (
          <g
            key={value instanceof Date ? value.toISOString() : value}
            transform={`translate(${isVertical ? 0 : offset},${isVertical ? offset : 0})`}
          >
            <line
              transform={`rotate(${isVertical ? 0 : rotationAngle})`}
              x2={isVertical ? -tickLineLength : tickLineLength} // Adjust line length for vertical axis
              y2={isVertical ? 0 : tickLineLength}  // Adjust line length for horizontal axis
              stroke="currentColor"
            />
            <text
              className='font-mono'
              style={{
                fontSize: `${fontSize}px`,
                textAnchor: isVertical ? "end" : "middle", // Align text according to orientation
                transform: isVertical ? `translateX(${tickTextOffsetX}px) translateY(${tickTextOffsetY}px)` : `translateY(${horizontalTextYOffset}px)`,
              }}
            >
              {formattedValue || value.toString()}
            </text>
          </g>
        ))}
      </g>
    </g>
  );
};

export default AxisComponent;
